@@ -1,17 +1,16 @@
 package DateTime::Indic::Chandramana;
 
-# $Id: Chandramana.pm 18 2009-07-20 20:39:26Z jaldhar.vyas $
+# $Id$
 
 use warnings;
 use strict;
 use Carp qw/ carp croak /;
-use DateTime::Event::Lunar;
 use DateTime::Indic::Utils qw/ epoch sidereal_year sidereal_month
-  lunar_on_or_before solar_longitude saura_rashi saura_varsha tithi_at_dt
+  lunar_on_or_before newmoon saura_rashi saura_varsha solar_longitude
+  tithi_at_dt
   /;
 use DateTime::Event::Sunrise;
 use DateTime::Util::Calc qw/ amod dt_from_moment mod search_next /;
-use Memoize;
 use Params::Validate qw/ validate BOOLEAN SCALAR OBJECT UNDEF /;
 use POSIX qw/ floor /;
 
@@ -113,14 +112,15 @@ one complete phase cycle of the Moon. Some calendars use amAsanta mAsa which
 end on the day of the new moon.  Others use pUrNimAnta mAsa which end on the
 day of the full moon.
 
-The Sanskrit names of the mAsa are:
+The Sanskrit names of the mAsa and their approximate correspondence to 
+Western months are:
 
-  1  chaitra       7  ashvina 
-  2  vaishAkha     8  kArtika
-  3  jyeShTa       9  mArgashIrasa 
-  4  AShADha       10 pauSha
-  5  shrAvaNa      11 mAgha
-  6  bhAdrapada    12 phAlguna
+  1  chaitra (March-April)            7  ashvina (September-October)
+  2  vaishAkha (April-May)            8  kArtika (October-November)
+  3  jyeShTa (May-June)               9  mArgashIrasa (November-December)
+  4  AShADha (June-July)              10 pauSha (December-January)
+  5  shrAvaNa (July-August)           11 mAgha (January-February)
+  6  bhAdrapada (August-September)    12 phAlguna (February-March)
 
 Some calendars start from a mAsa other than chaitra.  Nevertheless chaitra 
 would still be considered the "first" month despite not being the first month 
@@ -179,7 +179,23 @@ stretches over two sunrises.  This is called a vR^iddha ("large") tithi.  In
 this case, both days have the same number and name.  The first is prefixed 
 adhika or "leap".
 
-=head2 Avantika
+=head2 Solar Day (vAra)
+
+The Indian day does not start after midnight but at sunrise.  The period 
+from sunrise to sunset is called ahasa ("day") and the period from sunset to 
+the next sunrise is called rAtra ("night.") Together they make one ahorAtra or
+vAra.  Each vAra has a name in a seven-day cycle. Thr Sanskrit names of the
+vAra are:
+
+  1 ravivAra ("day of the Sun" i.e. Sunday)
+  2 somavAra ("day of the Moon" i.e. Monday)
+  3 ma~ngalavAra ("day of Mars" i.e. Tuesday)
+  4 budhavAra ("day of Mercury" i.e. Wednesday)
+  5 guruvAra  ("day of Jupiter" i.e. Thursday)
+  6 shukravAra ("day of Venus" i.e. Friday)
+  7 shanivAra ("day of Saturn" i.e. Saturday)
+
+=head2 Latitude, Longitude, and Avantika
 
 In order to know the correct chandramAna date, you have to know the time of 
 sunrise and this varies depending on where on Earth you are.  In this module
@@ -242,6 +258,12 @@ expressed as decimal degrees.  Negative values are used for longitudes west of
 Greenwich so the allowable range for this argument is from -180 to 180.  
 Defaults to 75.76, the longitude of avantika.
 
+=item * time_zone
+
+Time zone for which the panchanga is to be calculated.  It can either be a
+timezone name accepted by L<DateTime::TimeZone> or a L<DateTime::TimeZone> 
+object.  Defaults to 'Asia/Kolkata' (Indian Standard Time.)
+
 =back
 
 =cut
@@ -310,6 +332,10 @@ sub new {
                       sub { ( $_[0] >= -180 && $_[0] < 180 ) },
                 },
             },
+            time_zone => {
+                type    => SCALAR | OBJECT,
+                default => 'Asia/Kolkata',    # Indian Standard Time
+            },
         }
     );
 
@@ -320,20 +346,26 @@ sub new {
       ? $self->{tithi} + 15
       : $self->{tithi};
 
+    $self->{sun} = DateTime::Event::Sunrise->new(
+        latitude  => $self->{latitude},
+        longitude => $self->{longitude},
+        altitude  => 0,
+    );
+
+    ( $self->{rd_days}, $self->{rd_secs}, $self->{rd_nanosecs} ) =
+      $self->_fixed_from_lunar;
+
+    $self->{vara} = amod( $self->{rd_days} - epoch - 1, 7 );
+
     return $self;
 }
 
 sub _fixed_from_lunar {
     my ($self) = @_;
 
-    memoize('_lunar_from_fixed');
-
     my $approx =
       epoch +
-      sidereal_year *
-      ( $self->{varsha} +
-          $self->_era +
-          ( $self->{masa} - ( $self->_masa_offset > 1 ? 0 : 1 ) ) / 12.0 );
+      sidereal_year * ( $self->{varsha} + $self->_era + $self->{masa} / 12.0 );
 
     if ( $self->{masa} < $self->_masa_offset ) {
         $approx += sidereal_year;
@@ -342,19 +374,24 @@ sub _fixed_from_lunar {
     my $s = floor(
         $approx - ( 1.0 / 360.0 ) * sidereal_year * (
             mod(
-                solar_longitude( dt_from_moment($approx) ) -
-                  ( $self->{masa} - ( $self->_masa_offset > 1 ? 0 : 1 ) ) * 30 +
-                  180,
+                solar_longitude(
+                    dt_from_moment($approx)->set_time_zone( $self->{time_zone} )
+                      ->jd
+                  ) - $self->{masa} * 30 + 180,
                 360
               ) - 180
         )
     );
 
-    my $k = tithi_at_dt( dt_from_moment( $s + ( 1.0 / 4.0 ) ) );
+    my $k =
+      tithi_at_dt( dt_from_moment( $s + ( 1.0 / 4.0 ) )
+          ->set_time_zone( $self->{time_zone} ) );
 
     my $x;
-    my $mid = $self->_lunar_from_fixed( dt_from_moment( $s - 15 ),
-        $self->{latitude}, $self->{longitude} );
+
+    my $mid = $self->_lunar_from_fixed(
+        dt_from_moment( $s - 15 )->set_time_zone( $self->{time_zone} ),
+        $self->{sun} );
     if (
         $mid->{masa} < $self->{masa}
         || ( $mid->{adhikamasa}
@@ -370,41 +407,35 @@ sub _fixed_from_lunar {
     my $est = $s + $self->{lunar_day} - $x;
 
     my $tau = $est - mod(
-        tithi_at_dt( dt_from_moment( $est + ( 1.0 / 4.0 ) ) ) -
-          $self->{lunar_day} +
-          15,
+        tithi_at_dt(
+            dt_from_moment( $est + ( 1.0 / 4.0 ) )
+              ->set_time_zone( $self->{time_zone} )
+          ) - $self->{lunar_day} + 15,
         30
     ) + 15;
 
-    my $date = dt_from_moment($tau);
+    my $date = dt_from_moment($tau)->set_time_zone( $self->{time_zone} );
 
     search_next(
         base  => $date,
         check => sub {
-            return lunar_on_or_before(
-                $self,
-                $self->_lunar_from_fixed(
-                    $_[0], $self->{latitude}, $self->{longitude},
-                ),
+            return lunar_on_or_before( $self,
+                $self->_lunar_from_fixed( $_[0], $self->{sun}, ),
             );
         },
         next => sub { $_[0]->add( days => 1 ); },
     );
 
-    return ( $date->utc_rd_values )[0];
+    return $date->utc_rd_values;
 }
 
 sub _lunar_from_fixed {
-    my ( $self, $dt, $latitude, $longitude ) = @_;
+    my ( $self, $dt, $sun ) = @_;
 
     my $result = {};
 
-    my $sun = DateTime::Event::Sunrise->new(
-        latitude  => $latitude,
-        longitude => $longitude,
-        altitude  => 0,
-    );
-    my $suryodaya = $sun->sunrise_datetime($dt);
+    my $suryodaya =
+      $sun->sunrise_datetime($dt)->set_time_zone( $dt->time_zone );
 
     $result->{lunar_day} = tithi_at_dt($suryodaya);
 
@@ -425,14 +456,8 @@ sub _lunar_from_fixed {
     }
 
     # masa and adhikamasa
-    my $pnm = DateTime::Event::Lunar->new_moon_before(
-        datetime     => $suryodaya,
-        on_or_before => 1
-    );
-    my $nnm = DateTime::Event::Lunar->new_moon_after(
-        datetime    => $suryodaya,
-        on_or_after => 1
-    );
+    my $pnm = newmoon( $suryodaya->jd, 0 );
+    my $nnm = newmoon( $suryodaya->jd, 1 );
     my $solarmonth     = saura_rashi($pnm);
     my $nextsolarmonth = saura_rashi($nnm);
 
@@ -475,9 +500,30 @@ sub clone {
 =head3 from_object
 
 Builds a C<DateTime::Calendar::Chandramana> object from another I<DateTime> 
-object.  This function takes an C<object> parameter and optionally C<locale>. 
-For example:
+object.  This function takes the following parameters:
 
+=over 4
+
+=item * object
+
+a DateTime API compatible object.  (Basically this means it supports 
+L<utc_rd_Values>.)  This parameter is required.
+
+=item * latitude
+
+The latitude of the point for which the date is to be calculated 
+expressed as decimal degrees.  Negative values are used for latitudes south of 
+the equator so the allowable range for this argument is from -180 to 180.  
+Defaults to 23.15, the latitude of avantika.
+
+=item * longitude
+
+The longitude of the point for which the panchanga is to be calculated 
+expressed as decimal degrees.  Negative values are used for longitudes west of 
+Greenwich so the allowable range for this argument is from -180 to 180.  
+Defaults to 75.76, the longitude of avantika.
+
+=back
 
 =cut
 
@@ -515,9 +561,13 @@ sub from_object {
         }
     );
 
-    my $results =
-      $class->_lunar_from_fixed( $args{object}, $args{latitude},
-        $args{longitude}, );
+    my $sun = DateTime::Event::Sunrise->new(
+        latitude  => $args{latitude},
+        longitude => $args{longitude},
+        altitude  => 0,
+    );
+
+    my $results = $class->_lunar_from_fixed( $args{object}, $sun, );
 
     my $newobj = $class->new(
         varsha      => $results->{varsha},
@@ -528,6 +578,7 @@ sub from_object {
         adhikatithi => $results->{adhikatithi},
         latitude    => $args{latitude},
         longitude   => $args{longitude},
+        time_zone   => $args{object}->time_zone,
     );
     return $newobj;
 }
@@ -561,6 +612,10 @@ specifiers are allowed in the format string:
 
 =item * %T Equivalent to L<tithi_name>.
 
+=item * %v Equivalent to L<vara_abbrev>.
+
+=item * %V Equivalent to L<vara_name>.
+
 =item * %w Equivalent to L<tithi>.
 
 =item * %x Equivalent to '%y %A %M %P %L %w'
@@ -598,6 +653,10 @@ my %formats = (
 
     'T' => sub { $_[0]->tithi_name },
 
+    'v' => sub { $_[0]->vara_abbrev },
+
+    'V' => sub { $_[0]->vara_name },
+
     'w' => sub { $_[0]->tithi },
 
     'x' => sub {
@@ -662,12 +721,6 @@ nanoseconds.  See L<DateTime> for more details.
 sub utc_rd_values {
     my ($self) = @_;
 
-    if ( !exists $self->{rd_days} ) {
-        $self->{rd_days}     = $self->_fixed_from_lunar;
-        $self->{rd_secs}     = 0;
-        $self->{rd_nanosecs} = 0;
-    }
-
     return ( $self->{rd_days}, $self->{rd_secs}, $self->{rd_nanosecs} || 0 );
 }
 
@@ -743,7 +796,7 @@ sub adhikamasa_name {
 
 =head3 masa
 
-Returns the mAsa as a number from 1 to 12
+Returns the mAsa as a number from 1 (chaitra) to 12
 
 =cut
 
@@ -902,10 +955,46 @@ sub tithi_name {
     return $tithi_nama[ $self->{tithi} - 1 ];
 }
 
+=head3 vara
+
+Returns the vAra as a number from 1 (ravivAra) to 7.
+
+=cut
+
+sub vara {
+    my ($self) = @_;
+
+    return $self->{vara};
+}
+
+=head3 vara_abbrev
+
+Returns the abbreviated vAra name.
+
+=cut
+
+sub vara_abbrev {
+    my ($self) = @_;
+
+    return $vara_abbrev[ $self->{vara} - 1 ];
+}
+
+=head3 vara_name
+
+Returns the full vAra name.
+
+=cut
+
+sub vara_name {
+    my ($self) = @_;
+
+    return $vara_nama[ $self->{vara} - 1 ];
+}
+
 =head1 BUGS
 
 Please report any bugs or feature requests through the web interface at
-<http://code.google.com/p/panchanga/issues/list>. I
+L<https://github.com/jaldhar/panchanga/issues>. I
 will be notified, and then you’ll automatically be notified of progress
 on your bug as I make changes. B<Please do not use rt.cpan.org!.>
 
@@ -926,13 +1015,9 @@ You can also look for information at:
 
 =over 4
 
-=item * This projects web site
+=item * This projects git source code repository
 
-L<http://code.google.com/p/panchanga/>
-
-=item * This projects (read-only) subversion source code repository
-
-L<http://panchanga.googlecode.com/svn/perl/>
+L<https://github.com/jaldhar/panchanga/tree/master/perl>
 
 =item * AnnoCPAN: Annotated CPAN documentation
 
@@ -960,8 +1045,17 @@ Jaldhar H. Vyas, C<< <jaldhar at braincells.com> >>
 
 Copyright (C) 2009, Consolidated Braincells Inc.
 
-This library is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This program is free software; you can redistribute it and/or modify it under
+the terms of either:
+
+=over 4
+
+=item * the GNU General Public License as published by the Free Software
+Foundation; either version 2, or (at your option) any later version, or
+
+=item * the Artistic License version 2.0.
+
+=back
 
 The full text of the license can be found in the LICENSE file included
 with this distribution.
